﻿//===============================================================================
// Microsoft patterns & practices Enterprise Library
// Security Application Block
//===============================================================================
// Copyright © Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.Unity;
using Microsoft.Practices.ObjectBuilder2;

namespace Microsoft.Practices.EnterpriseLibrary.Security.AzMan.Configuration.Unity
{
	/// <summary>
	/// This type supports the Enterprise Library infrastructure and is not intended to be used directly from your code.
	/// Represents the process to create the container policies required to create a <see cref="AzManAuthorizationProvider"/>.
	/// </summary>
	public class AzManAuthorizationProviderPolicyCreator : IContainerPolicyCreator
	{
		void IContainerPolicyCreator.CreatePolicies(
			IPolicyList policyList,
			string instanceName,
			ConfigurationElement configurationObject,
			IConfigurationSource configurationSource)
		{
			AzManAuthorizationProviderData castConfigurationObject = (AzManAuthorizationProviderData)configurationObject;

			new PolicyBuilder<AzManAuthorizationProvider, AzManAuthorizationProviderData>(
				instanceName,
				castConfigurationObject,
				c => new AzManAuthorizationProvider(
					castConfigurationObject.StoreLocation,
					castConfigurationObject.Application,
					castConfigurationObject.AuditIdentifierPrefix,
					castConfigurationObject.Scope))

				.AddPoliciesToPolicyList(policyList);
		}
	}
}
